function xhat = basisPursuitEstFast(y,A,gam,x);
% Min (1/2)*||y-A*x||^2_2 + gam*||x||_1 

% Get dimensions
[m,n] = size(A);

nit = 100;
Jiter = zeros(nit,2);
a = 4;
xhat = zeros(n,1);
P = A'*A;
step = 0.1;
update = 1;
prt = 0;
for it = 1:nit
    
    % Compute value, gradient and Hessian
    if (update)
        e = y-A*xhat;
        J0 = 0.5*sum(abs(e).^2) + (gam/a)*sum(log(cosh(a*xhat)));
        J0act  = 0.5*sum(abs(e).^2) + gam*sum(abs(xhat));
        Jgrad = -A'*e + gam*tanh(a*xhat);
        H = P + diag(4*a*gam./cosh(a*xhat)); 
        delx = H \ Jgrad;
    end
    
    % Record current value
    Jiter(it,:) = [J0act J0];
    
    % Compute step 
    dx = -step*delx;
    x1 = xhat + dx;
    dJest = Jgrad'*dx;
    
    % Evaluate new point
    e1 = y-A*x1;
    J1 = 0.5*sum(abs(e1).^2) + (gam/a)*sum(log(cosh(a*x1)));
    
    % Check if condition passes
    if (J1-J0 < 0.5*dJest)
        if (prt)
            fprintf(1,'%d J=%12.4e %12.4e a=%5.1f step=%12.4e pass\n', it, Jiter(it,:), a, step);
        end
        step = step*2;
        xhat = x1;
        update = 1;
        
        % Test if a should be increased
        if (J1 > J0 - 0.01)
            a = min(64,2*a);
        end
        
    else
        if (prt)
            fprintf(1,'%d J=%12.4e %12.4e a=%5.1f step=%12.4e fail\n', it, Jiter(it,:), a, step);
        end
        step = step/2;
        update = 0;
    end
    
    
end

% Compute change in cost function by removing component
xhat1 = xhat;
nit1 = 10;
J1 = zeros(nit1,1);
Asq = sum(abs(A).^2)';
Jnzold = zeros(n,1);

for it1 = 1:nit1
    % Measure cost function and save if current is lower than new
    J1(it1) = 0.5*sum(abs(y-A*xhat1).^2) + gam*sum(abs(xhat1));
    if (J1(it1) < J0act)
        xhat = xhat1;
        J0act = J1(it1);
    end
    
    % Exit if cost function is growing
    if (J1(it1) > 2*J0act)
        if (prt)
            fprintf(1,'%d J=%12.4e Optimal solution not found\n', it1, J1(it1));     
        end
        break;
    end
    
    % Generate new test point
    z = A'*e + xhat1.*Asq;
    Jnz = (abs(z) > gam);
    if (sum(Jnz) >= m-1)
        break;
    end
    
    if (all(Jnz == Jnzold))
        if (prt)
            fprintf(1,'%d J=%12.4e Optimal solution found\n', it1, J1(it1)); 
        end
        break;
    else
        Jnzold = Jnz;
    end
    I = find(Jnz);
    s = sign(z(I));
    AI = A(:,I);
    xhat1 = zeros(n,1);
    xhat1(I) = (AI'*AI) \ (AI'*y - s*gam);    
end

